@@ -67,6 +67,14 @@ def main(args=None):
                         default=os.environ.get("OS_ENDPOINT_TYPE",
                                                "publicURL"),
                         help="Defaults to env[OS_ENDPOINT_TYPE] or publicURL")
+    parser.add_argument("--os-cert", type=str, metavar='<certificate>',
+                        default=os.environ.get("OS_CERT"),
+                        help="User's certificate. "
+                             "Defaults to env[OS_CERT].")
+    parser.add_argument("--os-key", type=str, metavar='<key>',
+                        default=os.environ.get("OS_KEY"),
+                        help="User's key. "
+                             "Defaults to env[OS_KEY].")
     parser.add_argument('--exclude-servers', action='store_true',
                         default=False,
                         help="Do not export in template server resources")
@@ -80,11 +88,15 @@ def main(args=None):
                         default=False,
                         help="In addition to template, generate Heat "
                              "stack data file.")
+    parser.add_argument('--extract-ports', action='store_true',
+                        default=False,
+                        help="Export the tenant network ports")
 
     args = parser.parse_args()
     flame = client.Client(args.username, args.password,
                           args.project, args.auth_url,
                           args.os_auth_token,
+                          cert=args.os_cert, key=args.os_key,
                           region_name=args.region,
                           endpoint_type=args.endpoint_type,
                           insecure=args.insecure)
@@ -92,10 +104,8 @@ def main(args=None):
     template.extract_vm_details(args.exclude_servers,
                                 args.exclude_volumes,
                                 args.exclude_keypairs,
-                                args.generate_stack_data)
+                                args.generate_stack_data,
+                                args.extract_ports)
     template.extract_data()
     print("### Heat Template ###")
-    print(template.heat_template())
-    if args.generate_stack_data:
-        print("### Stack Data ###")
-        print(template.stack_data_template())
+    print(template.heat_template_and_data())
