@@ -24,7 +24,9 @@
 
 import logging
 
+import concurrent.futures
 import netaddr
+import six
 import yaml
 
 from flameclient import managers
@@ -105,12 +107,14 @@ class Resource(object):
 class TemplateGenerator(object):
 
     def __init__(self, username, password, tenant_name, auth_url,
-                 auth_token=None, insecure=False, endpoint_type='publicURL',
-                 region_name=None):
+                 auth_token=None, cert=None, key=None, insecure=False,
+                 endpoint_type='publicURL', region_name=None):
+        self.thread_pool = concurrent.futures.ThreadPoolExecutor(10)
         self.generate_data = False
         self._setup_templates()
-        self._setup_managers(username, password, tenant_name, auth_url,
-                             insecure, endpoint_type, region_name, auth_token)
+        self._setup_managers(username, password, tenant_name, auth_url, cert,
+                             key, insecure, endpoint_type, region_name,
+                             auth_token)
 
     def _setup_templates(self):
         self.template = yaml.load(template_skeleton)
@@ -121,12 +125,12 @@ class TemplateGenerator(object):
         self.stack_data['resources'] = {}
 
     def _setup_managers(self, username, password, tenant_name, auth_url,
-                        insecure, endpoint_type, region_name=None,
-                        auth_token=None):
+                        insecure, endpoint_type, cert=None, key=None,
+                        region_name=None, auth_token=None):
         self.keystone = managers.KeystoneManager(
             username, password,
             tenant_name,
-            auth_url, insecure,
+            auth_url, cert, key, insecure,
             endpoint_type,
             region_name=region_name,
             auth_token=auth_token
@@ -136,33 +140,71 @@ class TemplateGenerator(object):
         self.nova = managers.NovaManager(self.keystone)
         self.cinder = managers.CinderManager(self.keystone)
 
+    def async_fetch_data(self, fetch_map):
+        """Call the methods in fetch_map in parallel and filter the results.
+
+        fetch_map is a dict of the form :
+            {resource_type, (fetch_method, filter_method), ...}
+
+        This method returns an iterator on the filtered results.
+        The iterator items are of the form (res_type, result)
+        """
+
+        future_res = {}
+        with self.thread_pool as tp:
+            for res_type, (method, data_filter) in six.iteritems(fetch_map):
+                future_res[tp.submit(method)] = res_type
+            for res_available in concurrent.futures.as_completed(future_res):
+                res = res_available.result()
+                res_type = future_res[res_available]
+                yield res_type, fetch_map[res_type][1](res)
+
+    def order_ports(self):
+        for i, port in self.ports.values():
+            for fixed_ip in port['fixed_ips']:
+                ip_subnet = self.subnets[fixed_ip['subnet_id']][1]
+                pools = ip_subnet.get('allocation_pools')
+                if pools:
+                    pools_starts = [pool['start'] for pool in pools]
+                    if fixed_ip['ip_address'] in pools_starts:
+                        # Its the first port of the subnet
+                        ip_subnet['first_port'] = port
+
     def extract_vm_details(self, exclude_servers, exclude_volumes,
-                           exclude_keypairs, generate_data):
+                           exclude_keypairs, generate_data,
+                           extract_ports=False):
         self.exclude_servers = exclude_servers
         self.exclude_volumes = exclude_volumes
         self.exclude_keypairs = exclude_keypairs
         self.generate_data = generate_data
-
-        self.subnets = self.build_data(self.neutron.subnet_list())
-        self.networks = self.build_data(self.neutron.network_list())
-        self.routers = self.neutron.router_list()
-        self.secgroups = self.build_data(self.neutron.secgroup_list())
-        self.floatingips = self.neutron.floatingip_list()
-        self.ports = self.build_data(self.neutron.port_list())
+        self.extract_ports = extract_ports
         self.external_networks = []
+        fetch_map = {
+            'subnets': (self.neutron.subnet_list, self.build_data),
+            'networks': (self.neutron.network_list, self.build_data),
+            'routers': (self.neutron.router_list, lambda x: x),
+            'secgroups': (self.neutron.secgroup_list, self.build_data),
+            'servergroups': (self.nova.servergroup_list, self.build_data),
+            'floatingips': (self.neutron.floatingip_list, lambda x: x),
+            'ports': (self.neutron.port_list, self.build_data),
+        }
 
         if not exclude_keypairs:
-            self.keys = dict(
-                (key.name, (index, key))
-                for index, key in enumerate(self.nova.keypair_list()))
+            fetch_map['keys'] = (self.nova.keypair_list,
+                                 lambda l: {key.name: (index, key) for
+                                            index, key in enumerate(l)})
 
         if not exclude_servers:
-            self.flavors = self.build_data(self.nova.flavor_list())
-            self.servers = self.build_data(self.nova.server_list())
+            fetch_map['flavors'] = (self.nova.flavor_list, self.build_data)
+            fetch_map['servers'] = (self.nova.server_list, self.build_data)
 
         if (not exclude_volumes or
                 (exclude_volumes and not exclude_servers)):
-            self.volumes = self.build_data(self.cinder.volume_list())
+            fetch_map['volumes'] = (self.cinder.volume_list, self.build_data)
+
+        for res_type, result in self.async_fetch_data(fetch_map):
+            self.__setattr__(res_type, result)
+        self.order_ports()
 
     def build_data(self, data):
         if not data:
@@ -263,6 +305,22 @@ class TemplateGenerator(object):
     def get_subnet_resource_name(self, subnet_id):
         return "subnet_%d" % self.subnets[subnet_id][0]
 
+    def get_server_resource_name(self, device_id):
+        return "server_%d" % self.servers[device_id][0]
+
+    def get_router_resource_name(self, device_id):
+        return "router_%d" % self.routers[device_id][0]
+
+    def get_secgroup_resource_name(self, secgroup_id):
+        return "security_group_%d" % self.secgroups[secgroup_id][0]
+
+    def get_ports_for_server(self, server_id):
+        ports = []
+        for n, port in self.ports.values():
+            if port['device_id'] == server_id:
+                ports.append("port_%d" % n)
+        return ports
+
     def _extract_subnets(self):
         resources = []
         for n, subnet in self.subnets.values():
@@ -285,11 +343,56 @@ class TemplateGenerator(object):
             resources.append(resource)
         return resources
 
+    def _extract_ports(self):
+        resources = []
+        resources_dict = {}
+        self.dhcp_fixed_ips = {}
+        for n, port in self.ports.values():
+            fixed_ips = []
+            for fixed_ip_dict in port['fixed_ips']:
+                subnet_id = fixed_ip_dict['subnet_id']
+                subnet_name = self.get_subnet_resource_name(subnet_id)
+                fixed_ip_resource = {u'subnet_id':
+                                     {'get_resource': subnet_name},
+                                     u'ip_address': fixed_ip_dict['ip_address']
+                                     }
+                fixed_ips.append(fixed_ip_resource)
+
+                if port['device_owner'] == 'network:dhcp':
+                    # Add the fixed ip to the dhcp_fixed_ips list
+                    dhcp_ips = self.dhcp_fixed_ips.setdefault(subnet_name, [])
+                    dhcp_ips.append(fixed_ip_dict['ip_address'])
+            if not port['device_owner'].startswith('compute:'):
+                # It's not a server, skip it!
+                continue
+            net_name = self.get_network_resource_name(port['network_id'])
+            properties = {
+                'network_id': {'get_resource': net_name},
+                'admin_state_up': port['admin_state_up'],
+                'fixed_ips': fixed_ips,
+                'mac_address': port['mac_address'],
+                'device_owner': port['device_owner'],
+            }
+            if port['name'] != '':
+                # This port has a name
+                properties['name'] = port['name']
+            resource = Resource("port_%d" % n, 'OS::Neutron::Port',
+                                port['id'], properties)
+            security_groups = self.build_port_secgroups(resource, port)
+            properties['security_groups'] = security_groups
+
+            resources.append(resource)
+            resources_dict[port['id']] = resource
+
+        return resources
+
     def _build_rules(self, rules):
         brules = []
         for rule in rules:
             if rule['protocol'] == 'any':
                 del rule['protocol']
+                del rule['port_range_min']
+                del rule['port_range_max']
             rg_id = rule['remote_group_id']
             if rg_id is not None:
                 rule['remote_mode'] = "remote_group_id"
@@ -327,6 +430,16 @@ class TemplateGenerator(object):
             resources.append(resource)
         return resources
 
+    def _extract_servergroups(self):
+        resources = []
+        for n, servergroup in self.servergroups.values():
+            properties = {'name': servergroup.name,
+                          'policies': servergroup.policies}
+            resource = Resource("servergroup_%d" % n, 'OS::Nova::ServerGroup',
+                                servergroup.id, properties)
+            resources.append(resource)
+        return resources
+
     def _extract_keys(self):
         resources = []
         for n, key in self.keys.values():
@@ -360,6 +473,31 @@ class TemplateGenerator(object):
 
         return security_groups
 
+    def build_port_secgroups(self, resource, port):
+        security_groups = []
+        port_secgroups = [self.secgroups[sgid][1]
+                          for sgid in port['security_groups']]
+
+        secgroup_default_parameter = None
+        for secgr in port_secgroups:
+            if secgr['name'] == 'default' and self.generate_data:
+                if not secgroup_default_parameter:
+                    port_res_name = 'port_%d' % self.ports[port['id']][0]
+                    param_name = "%s_default_security_group" % port_res_name
+                    description = ("Default security group for port %s" %
+                                   port['name'])
+                    default = secgr['id']
+                    resource.add_parameter(param_name, description,
+                                           default=default)
+                    secgroup_default_parameter = {'get_param': param_name}
+                security_groups.append(secgroup_default_parameter)
+            else:
+                resource_name = ("security_group_%d" %
+                                 self.secgroups[secgr['id']][0])
+                security_groups.append({'get_resource': resource_name})
+
+        return security_groups
+
     def build_networks(self, addresses):
         networks = []
         for net_name in addresses:
@@ -374,6 +512,9 @@ class TemplateGenerator(object):
                             networks.append({'network': {'get_resource': net}})
         return networks
 
+    def get_servergroup_resource_name(self, servergroup_id):
+        return "servergroup_%d" % self.servergroups[servergroup_id][0]
+
     def _extract_servers(self):
         resources = []
         for n, server in self.servers.values():
@@ -421,13 +562,19 @@ class TemplateGenerator(object):
                     resource_key = "key_%d" % self.keys[server.key_name][0]
                     properties['key_name'] = {'get_resource': resource_key}
 
-            security_groups = self.build_secgroups(resource, server)
-            if security_groups:
-                properties['security_groups'] = security_groups
+            if self.extract_ports:
+                ports = [{"port": {"get_resource": port}}
+                         for port in self.get_ports_for_server(server.id)]
+                if ports:
+                    properties['networks'] = ports
+            else:
+                security_groups = self.build_secgroups(resource, server)
+                if security_groups:
+                    properties['security_groups'] = security_groups
 
-            networks = self.build_networks(server.addresses)
-            if networks:
-                properties['networks'] = networks
+                networks = self.build_networks(server.addresses)
+                if networks:
+                    properties['networks'] = networks
 
             if server.metadata:
                 properties['metadata'] = server.metadata
@@ -457,6 +604,15 @@ class TemplateGenerator(object):
                 # block devices to an instance
                 properties['block_device_mapping_v2'] = server_volumes
 
+            # server-group
+            for servergroup in self.servergroups.values():
+                if server.id in servergroup[1].members:
+                    hint = {'group': {'get_resource':
+                            self.get_servergroup_resource_name
+                            (servergroup[1].id)}}
+                    properties['scheduler_hints'] = \
+                        hint
+
             resources.append(resource)
         return resources
 
@@ -478,20 +634,36 @@ class TemplateGenerator(object):
                                    default=default)
             resources.append(resource)
 
-            if not self.exclude_servers and ip['port_id']:
-                device = self.ports[ip['port_id']][1]['device_id']
-                if device and self.servers[device]:
-                    server = self.servers[device]
-                    server_resource_name = "server_%d" % server[0]
-                    properties = {
-                        'floating_ip': {'get_resource': ip_resource_name},
-                        'server_id': {'get_resource': server_resource_name}
-                    }
-                    resource = Resource("floatingip_association_%d" % n,
-                                        'OS::Nova::FloatingIPAssociation',
-                                        None,
-                                        properties)
-                    resources.append(resource)
+            if self.extract_ports and ip['port_id']:
+                port_number = self.ports[ip['port_id']][0]
+                port_resource_name = "port_%d" % port_number
+                properties = {
+                    'floatingip_id': {'get_resource': ip_resource_name},
+                    'port_id': {'get_resource': port_resource_name}
+                }
+                resource_id = ("%s:%s" %
+                               (ip['id'],
+                                ip['port_id']))
+                resource = Resource("floatingip_association_%d" % n,
+                                    'OS::Neutron::FloatingIPAssociation',
+                                    resource_id,
+                                    properties)
+                resources.append(resource)
+            else:
+                if not self.exclude_servers and ip['port_id']:
+                    device = self.ports[ip['port_id']][1]['device_id']
+                    if device and self.servers[device]:
+                        server = self.servers[device]
+                        server_resource_name = "server_%d" % server[0]
+                        properties = {
+                            'floating_ip': {'get_resource': ip_resource_name},
+                            'server_id': {'get_resource': server_resource_name}
+                        }
+                        resource = Resource("floatingip_association_%d" % n,
+                                            'OS::Nova::FloatingIPAssociation',
+                                            None,
+                                            properties)
+                        resources.append(resource)
         return resources
 
     def _extract_volumes(self):
@@ -548,9 +720,13 @@ class TemplateGenerator(object):
     def extract_data(self):
         resources = self._extract_routers()
         resources += self._extract_networks()
+        if self.extract_ports:
+            resources += self._extract_ports()
+
         resources += self._extract_subnets()
         resources += self._extract_secgroups()
         resources += self._extract_floating()
+        resources += self._extract_servergroups()
 
         if not self.exclude_keypairs:
             resources += self._extract_keys()
@@ -565,8 +741,13 @@ class TemplateGenerator(object):
             if self.generate_data:
                 self.stack_data['resources'].update(resource.stack_resource)
 
-    def heat_template(self):
-        return self.format_template(self.template)
+    def heat_template_and_data(self):
+        if self.generate_data:
+            out = self.stack_data.copy()
+            out['template'] = self.template
+            out['environment'] = {"parameter_defaults": {},
+                                  "parameters": {}}
 
-    def stack_data_template(self):
-        return self.format_template(self.stack_data)
+            return self.format_template(out)
+        else:
+            return self.format_template(self.template)
